@@ -55,7 +55,7 @@ def set_automated_followed_pool(
     delay_followbackers,
     pool="followedPool",
 ):
-    """ Generare a user list based on the InstaPy followed usernames """
+    """ Generate a user list based on the InstaPy followed usernames """
     pool_name = "{0}{1}_{2}.csv".format(logfolder, username, pool)
     automatedFollowedPool = {"all": {}, "eligible": {}}
     time_stamp = None
@@ -180,17 +180,19 @@ def get_following_status(
             return "UNAVAILABLE", None
     # wait until the follow button is located and visible, then get it
     try:
-        browser.find_element_by_xpath(
-            read_xpath(get_following_status.__name__, "follow_button_XP")
+        follow_button = browser.find_element_by_xpath(
+            read_xpath(get_following_status.__name__, "follow_span_XP_following")
         )
-        follow_button_XP = read_xpath(get_following_status.__name__, "follow_button_XP")
+        return "Following", follow_button
     except NoSuchElementException:
         try:
-            follow_button = browser.find_element_by_xpath(
-                read_xpath(get_following_status.__name__, "follow_span_XP_following")
+            browser.find_element_by_xpath(
+                read_xpath(get_following_status.__name__, "follow_button_XP")
             )
-            return "Following", follow_button
-        except:
+            follow_button_XP = read_xpath(
+                get_following_status.__name__, "follow_button_XP"
+            )
+        except Exception:
             return "UNAVAILABLE", None
     follow_button = explicit_wait(
         browser, "VOEL", [follow_button_XP, "XPath"], logger, 7, False
@@ -586,8 +588,8 @@ def unfollow(
 
 
 def follow_user(browser, track, login, user_name, button, blacklist, logger, logfolder):
-    """ Follow a user either from the profile page or post page or dialog
-    box """
+    """Follow a user either from the profile page or post page or dialog
+    box"""
     # list of available tracks to follow in: ["profile", "post" "dialog"]
 
     # check action availability
@@ -697,6 +699,7 @@ def get_users_through_dialog_with_graphql(
     jumps,
     logger,
     logfolder,
+    edge_followed_by,
 ):
 
     # TODO: simulation implmentation
@@ -714,13 +717,27 @@ def get_users_through_dialog_with_graphql(
             "return window._sharedData." "entry_data.ProfilePage[0]." "graphql.user.id"
         )
 
-    query_hash = get_query_hash(browser, logger)
+    # There are two query hash, one for followers and following, ie:
+    # t="c76146de99bb02f6415203be841dd25a",n="d04b0a864b4b54837c0d870b0e77e076"
+    if edge_followed_by:
+        # True: User requested session.follow_user_followers
+        edge_type = "edge_followed_by"
+    else:
+        # False: User requested session.follow_user_following
+        edge_type = "edge_follow"
+
+    query_hash = get_query_hash(browser, logger, edge_followed_by)
+
     # check if hash is present
     if query_hash is None:
         logger.info("Unable to locate GraphQL query hash")
+    else:
+        logger.info("GraphQL query hash: [{}]".format(query_hash))
 
-    graphql_query_URL = "view-source:https://www.instagram.com/graphql/query/?query_hash={}".format(
-        query_hash
+    graphql_query_URL = (
+        "view-source:https://www.instagram.com/graphql/query/?query_hash={}".format(
+            query_hash
+        )
     )
     variables = {
         "id": str(user_id),
@@ -735,8 +752,10 @@ def get_users_through_dialog_with_graphql(
     pre = browser.find_element_by_tag_name("pre")
     # set JSON object
     data = json.loads(pre.text)
-    # get all followers of current page
-    followers_page = data["data"]["user"]["edge_followed_by"]["edges"]
+    # get all followers or following of current page
+    # edge_type: used to check followers or following in JSON
+    #            "edge_followed_by" or "edge_follow"
+    followers_page = data["data"]["user"][str(edge_type)]["edges"]
     followers_list = []
 
     # iterate over page size and add users to the list
@@ -744,16 +763,14 @@ def get_users_through_dialog_with_graphql(
         # get follower name
         followers_list.append(follower["node"]["username"])
 
-    has_next_page = data["data"]["user"]["edge_followed_by"]["page_info"][
-        "has_next_page"
-    ]
+    has_next_page = data["data"]["user"][str(edge_type)]["page_info"]["has_next_page"]
 
     while has_next_page and len(followers_list) <= amount:
         # server call interval
         sleep(random.randint(2, 6))
 
         # get next page reference
-        end_cursor = data["data"]["user"]["edge_followed_by"]["page_info"]["end_cursor"]
+        end_cursor = data["data"]["user"][str(edge_type)]["page_info"]["end_cursor"]
 
         # url variables
         variables = {
@@ -763,21 +780,24 @@ def get_users_through_dialog_with_graphql(
             "first": 50,
             "after": end_cursor,
         }
+
         url = "{}&variables={}".format(graphql_query_URL, str(json.dumps(variables)))
         browser.get(url)
         pre = browser.find_element_by_tag_name("pre")
+
         # response to JSON object
         data = json.loads(pre.text)
 
         # get all followers of current page
-        followers_page = data["data"]["user"]["edge_followed_by"]["edges"]
+        followers_page = data["data"]["user"][str(edge_type)]["edges"]
+
         # iterate over page size and add users to the list
         for follower in followers_page:
             # get follower name
             followers_list.append(follower["node"]["username"])
 
         # check if there is next page
-        has_next_page = data["data"]["user"]["edge_followed_by"]["page_info"][
+        has_next_page = data["data"]["user"][str(edge_type)]["page_info"][
             "has_next_page"
         ]
 
@@ -835,7 +855,12 @@ def get_users_through_dialog_with_graphql(
 
     # get real amount
     followers_list = random.sample(followers_list, real_amount)
-    print(followers_list)
+
+    for i, user in enumerate(followers_list):
+        logger.info(
+            "To be followed: [{}/{}/{}]".format(i + 1, len(followers_list), user)
+        )
+
     return followers_list, []
 
 
@@ -1010,6 +1035,7 @@ def get_given_user_followers(
         return [], []
 
     channel = "Follow"
+    edge_followed_by = True
     person_list, simulated_list = get_users_through_dialog_with_graphql(
         browser,
         login,
@@ -1025,6 +1051,7 @@ def get_given_user_followers(
         jumps,
         logger,
         logfolder,
+        edge_followed_by,
     )
 
     return person_list, simulated_list
@@ -1044,6 +1071,21 @@ def get_given_user_following(
     logger,
     logfolder,
 ):
+    """
+    For the given username, follow who they follows.
+
+    :param browser: webdriver instance
+    :param login:
+    :param user_name: given username of account to follow
+    :param amount: the number of followers to follow
+    :param dont_include: ignore these usernames
+    :param randomize: randomly select from users' followers
+    :param blacklist:
+    :param follow_times:
+    :param logger: the logger instance
+    :param logfolder: the logger folder
+    :return: list of user's following
+    """
     user_name = user_name.strip().lower()
 
     user_link = "https://www.instagram.com/{}/".format(user_name)
@@ -1136,6 +1178,7 @@ def get_given_user_following(
         return [], []
 
     channel = "Follow"
+    edge_followed_by = False
     person_list, simulated_list = get_users_through_dialog_with_graphql(
         browser,
         login,
@@ -1151,6 +1194,7 @@ def get_given_user_following(
         jumps,
         logger,
         logfolder,
+        edge_followed_by,
     )
 
     return person_list, simulated_list
@@ -1203,8 +1247,8 @@ def dump_follow_restriction(profile_name, logger, logfolder):
 
 
 def follow_restriction(operation, username, limit, logger):
-    """ Keep track of the followed users and help avoid excessive follow of
-    the same user """
+    """Keep track of the followed users and help avoid excessive follow of
+    the same user"""
 
     try:
         # get a DB and start a connection
@@ -1474,8 +1518,8 @@ def get_user_id(browser, track, username, logger):
 
 
 def verify_username_by_id(browser, username, person, person_id, logger, logfolder):
-    """ Check if the given user has changed username after the time of
-    followed """
+    """Check if the given user has changed username after the time of
+    followed"""
     # try to find the user by ID
     if person_id is None:
         person_id = load_user_id(username, person, logger, logfolder)
